import java.io.*;
import java.net.*;
import java.awt.Color;
import java.util.Vector;

public class IFSParser extends StreamTokenizer {
	private static final boolean DEBUG = false;
	IFSImage currentImg;
	Vector images = new Vector();
	boolean valid_eq=false,
				valid_color=false,
				valid_leftbottom=false,
				valid_righttop=false;
	
	public IFSParser(Reader buffer){
		super(buffer);
	}
	
	private boolean skipNulls() throws IOException,EOFException {
  		do nextToken(); while (sval == null & ttype!=TT_EOF);
  		if (ttype==TT_EOF) throw new EOFException();
  		  debug("Esco da skipNulls con "+sval);
  		return true;
   }
  
   private boolean syntaxError(String msg){
   	System.out.println("Syntax error at line "+lineno()+": "+msg);
		return false;
	}
	
	
	private final boolean getImageHeader() throws IOException,EOFException {
		// Cerco una definizione di immagine IFS	
		nextToken();
		if (ttype==TT_EOF) throw new EOFException();
		if (ttype!=TT_WORD) syntaxError("Keyword IFSIMAGE expected");
		else if (!sval.equals("ifsimage")) syntaxError("Keyword IFSIMAGE expected");
		
		  debug("Trovata!");
		// Cerco il titolo
		lowerCaseMode(false);
		nextToken();
		if (ttype==TT_EOF) throw new EOFException();
		if (ttype!=TT_WORD) return syntaxError("A word defining image title expected. Ex: IFSIMAGE fooimage");

		currentImg.title = sval;
		  debug("Title: "+sval);
		lowerCaseMode(true);  
		return true;  
	} 
	
	private final boolean getEquation() throws IOException,EOFException{
		double n[] = new double[13]; 
		int i;
		 
		debug("Ho trovato EQ!");
		nextToken();
		if (ttype == TT_EOF) throw new EOFException();
		if (ttype != TT_WORD) return syntaxError("Keyword CF or PTS required after keyword EQ.");
		else if (sval.equals("cf")) {
			for (i=0;i<7;i++) {
				nextToken();
				if (ttype == TT_EOF) throw new EOFException();
				if (ttype != TT_NUMBER) return syntaxError("7 Space-separated double precision number (6 coefficients and probability factor) required after keywords EQ CF.");
				n[i]=nval;
		  	  	debug((i+1)+"-esimo valore: "+n[i]);
			}
			currentImg.code.addElement(new IFSItem(n[0],n[1],n[2],n[3],n[4],n[5],n[6]));
		}
		else if (sval.equals("pts")) {
			for (i=0;i<13;i++) {
				nextToken();
				if (ttype == TT_EOF) throw new EOFException();
				if (ttype != TT_NUMBER) return syntaxError("13 Space-separated double precision number (3x2 coords source points, 3x2 coords target points and probability factor) required after keywords EQ PTS.");
				n[i]=nval;
		  	  	debug((i+1)+"-esimo valore: "+n[i]);
			}
			currentImg.code.addElement(new IFSItem(n[0],n[1],n[2],n[3],n[4],n[5],n[6],n[7],n[8],n[9],n[10],n[11],n[12]));
		}
		else return syntaxError("Keyword CF or PTS required after keyword EQ.");
				
		return true;
	}
	
	private final boolean getColor()throws IOException,EOFException{
		nextToken(); 
		if (ttype == TT_EOF) throw new EOFException();
		if (ttype != TT_WORD){
			return syntaxError("A word defining a color required after keyword COLOR. Es. COLOR RED");
		}
		if (sval.equals("black")) currentImg.color=Color.black; 
  		else if (sval.equals( "blue")) currentImg.color=Color.blue;
  		else if (sval.equals( "cyan")) currentImg.color=Color.cyan; 
  		else if (sval.equals( "darkGray")) currentImg.color=Color.darkGray; 
  		else if (sval.equals( "gray")) currentImg.color=Color.gray; 
  		else if (sval.equals( "green")) currentImg.color=Color.green; 
  		else if (sval.equals( "lightGray")) currentImg.color=Color.lightGray;
  		else if (sval.equals( "magenta")) currentImg.color=Color.magenta; 
  		else if (sval.equals( "orange")) currentImg.color=Color.orange;
  		else if (sval.equals( "pink")) currentImg.color=Color.pink; 
  		else if (sval.equals( "red")) currentImg.color=Color.red; 
  		else if (sval.equals( "white")) currentImg.color=Color.white; 
  		else if (sval.equals( "yellow")) currentImg.color=Color.yellow;
  		else {
  			syntaxError("Color "+sval+" unknown. Setting to black.");
  			currentImg.color=Color.black;
  		}
  		  debug("Colore : "+sval);
  		return true;
   }
   
   private final boolean getLeftBottom() throws IOException,EOFException{
  		double nx,ny;
  		nextToken();
  		if (ttype == TT_EOF) throw new EOFException();
  		if (ttype != TT_NUMBER){
			return syntaxError("2 double precision numbers required after keyword LEFTBOTTOM. Es. LEFTBOTTOM 0.5 -42");
		}
  		nx = nval;
  		nextToken();
  		if (ttype == TT_EOF) throw new EOFException();
  		if (ttype != TT_NUMBER){
			return syntaxError("2 double precision numbers required after keyword LEFTBOTTOM. Es. LEFTBOTTOM 0.5 -42");
		}
  		ny = nval;
  								
		currentImg.leftbottom.setXY(nx,ny);
		  debug("LEFTBOTTOM : "+currentImg.leftbottom);
		return true;
  	}
	
	private final boolean getRightTop() throws IOException,EOFException{
  		double nx,ny;
  		nextToken();
  		if (ttype != TT_NUMBER){
			return syntaxError("2 double precision numbers required after keyword RIGHTTOP. Es. RIGHTTOP 0.5 -42");
		}
  		nx = nval;
  		nextToken();
  		if (ttype != TT_NUMBER){
			return syntaxError("2 double precision numbers required after keyword RIGHTTOP. Es. RIGHTTOP 0.5 -42");
		}
  		ny = nval;
  								
		currentImg.righttop.setXY(nx,ny);
		  debug("RIGHTTOP : " + currentImg.righttop);
		return true;  
	}
	
	private final void validateImage(){
		if (valid_eq & valid_color & valid_leftbottom & valid_righttop){
			images.addElement(currentImg);
			debug("Aggiunta immagine "+currentImg.title);
	   }
	}
	
	public final void parseIFSData(Vector images){
		boolean exitflag = false;
		this.images=images;
 	
		eolIsSignificant(false); 
		lowerCaseMode(true);
		slashSlashComments(true);
		
		debug ("Inizio a leggere ...");
			
		try{
			while (ttype != TT_EOF) {			
				currentImg = new IFSImage();
				getImageHeader();
				do {
				// Cerco le parole chiave riconosciute
					nextToken();
					if (ttype==TT_EOF) throw new EOFException();
					if (ttype!=TT_WORD) syntaxError("Keyword expected");
					else { // il token  una parola
						if(!sval.equals("eq") & !sval.equals("end") & !sval.equals("color") & !sval.equals("leftbottom") & !sval.equals("righttop")) syntaxError("keyword"+sval+"unknown");
						if (sval.equals("eq")) valid_eq |= getEquation();
						else if (sval.equals("color")) valid_color |= getColor();
  						else if (sval.equals("leftbottom")) valid_leftbottom |= getLeftBottom();
  						else if (sval.equals("righttop")) valid_righttop |= getRightTop();
  						else if (sval.equals("end")) exitflag=true;	
  					}
  				} while (!exitflag);
  			    	debug("IFSImage terminata");
  				 	validateImage();
  				  	exitflag=false;
  				  	valid_eq=false;
					valid_color=false;
					valid_leftbottom=false;
					valid_righttop=false;
			} 
			  debug ("File terminato");
		} catch (EOFException eofe) {
			validateImage();
		} catch (IOException ioe) {
			System.out.println("IOException: " + ioe);
		} 
					      
   }
	
	private void debug(String msg){
		if (DEBUG) System.out.println(msg);
	} 

}