import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.net.*;
import java.util.Vector;

public class Start extends java.applet.Applet implements Runnable {
		final static int WIDTH = 640;
		final static int HEIGHT = 350;
		final static boolean DEBUG = false;
		
		IFSGen out;
		Thread drawThread,manager;
		GUIManager gui = new GUIManager();
		String docbasedir;
 		Vector images = new Vector(); // vettore di IFSImage
	
		GUI mainFrame = new GUI("IFS Generator");
		Choice imgChoice = new Choice();
		TextField fileTextField = new TextField("",16);
		Button fileButton = new Button("Load");
		Button drawButton = new Button("Draw");
		Button clearButton = new Button("Clear");
		Label iterationsLabel = new Label("Iterations: ");	
		TextField iterationsTextField = new TextField("20000");		
  
  public final void loadIFSData(String url){	
		// Scarico dalla rete il file a cui punta url che contiene le immagini IFS
		try {
    		URL location = new URL(url);
    		debug ("URL : "+location.toString());	
  			BufferedReader data = new BufferedReader(new InputStreamReader(new DataInputStream(location.openStream())));
  		// Leggo il file	
  			IFSParser parser = new IFSParser(data);
  			parser.parseIFSData(images);
  		// Aggiorno la Choice list
  			imgChoice.removeAll();
  			imgChoice.addItem("Select an image:");
  			for (int i=0;i<images.size();i++){
  				imgChoice.addItem( ((IFSImage)images.elementAt(i)).title );
  			}
  			  				
  		} catch (MalformedURLException me) {
    		System.out.println("MalformedURLException: " + me);    	
    	} catch (IOException ioe) {
      	System.out.println("IOException: " + ioe);
    	} 
	}
  
public void init() {
  	out = new IFSGen(); 

// Trovo la directory in cui risiede l'applet 
	String docbase = this.getDocumentBase().toString();	
	docbasedir = docbase.substring(0,docbase.lastIndexOf("/"));
	  debug ("Document Base Directory : "+docbasedir);
  	loadIFSData(docbasedir+"/"+"ifsdata.txt");
	out.img = (IFSImage) ((IFSImage) images.elementAt(0)).clone();
  	//out.init((int) (WIDTH/2),HEIGHT);
  	
    /********************* LAYOUT ***********************/ 
		
		mainFrame.setSize(WIDTH,HEIGHT);
		mainFrame.setLayout(new XYLayout());	// posizionamento assoluto dei component
	  	mainFrame.add(out);
	  	mainFrame.add(imgChoice);
	  	mainFrame.add(fileTextField);
	  	mainFrame.add(fileButton);
	  	mainFrame.add(drawButton);
	  	mainFrame.add(clearButton);
	  	mainFrame.add(iterationsLabel);
	  	mainFrame.add(iterationsTextField);
	  	
	  	mainFrame.addWindowListener(gui);
	  	fileButton.addActionListener(gui);
	  	drawButton.addActionListener(gui);
	  	clearButton.addActionListener(gui);
	  	mainFrame.setVisible(true);
	  	mainFrame.repaint();
}
  
  public void start(){
  		if (drawThread == null) {
      	drawThread = new Thread(out,"IFSDraw");
      }
      if (manager == null) {
      	manager = new Thread(this,"Manager");
      	manager.setPriority(Thread.MIN_PRIORITY);
      	manager.start();
      }
  }
	
  
  
  public synchronized void run() {
  		while (true){
  			if (!out.drawing & !drawButton.getLabel().equals("Draw")) {
				drawButton.setLabel("Draw");
				debug("Setting label to draw!");
			}
  			manager.yield();
  		}
  }
  
  private void debug(String msg){
  	if (DEBUG) System.out.println(msg);
  }
  
  class GUI extends Frame {
  	
  		public GUI(String title){
  			super(title);
  		}
  		
  		public void update(Graphics g){
  			int outSize = ( (int) (getSize().width*0.5) < getSize().height) ? (int)(getSize().width*0.5) : getSize().height;
	  		out.init(outSize,(int)(outSize*0.8));
	  		out.setLocation(5,35);
	  		imgChoice.setLocation(5+out.getSize().width,35);
	  		fileTextField.setLocation(5+out.getSize().width,70);
	  		fileButton.setLocation(5+fileTextField.getPreferredSize().width+out.getSize().width,70);
	  		drawButton.setLocation(5+out.getSize().width,105);
	  		clearButton.setLocation(5+out.getSize().width+drawButton.getPreferredSize().width+5,105);
	  		iterationsLabel.setLocation(5+out.getSize().width,140);
	  		iterationsTextField.setLocation(5+out.getSize().width+iterationsLabel.getPreferredSize().width,140);
	  	
  		}
  }
  
  class GUIManager implements ActionListener,WindowListener {
	 public synchronized void actionPerformed(ActionEvent e){
	 	String source = e.getActionCommand();
	 	if (source.equals("Draw")){
	 		// Legge numero iterazioni
	 		 try {
	 			out.NITERATIONS=(new Integer(iterationsTextField.getText()) ).intValue();
	 		 } catch (NumberFormatException nfe) {
	 			// ripristina iterazioni iniziali sul TextField
	 			iterationsTextField.setText(String.valueOf(out.NITERATIONS));
	 		 }
	 		// Legge quale immagine disegnare e inizializza IFSGen 
	 		 if (imgChoice.getSelectedIndex() != 0){
	 			out.img = (IFSImage) ((IFSImage) images.elementAt(imgChoice.getSelectedIndex()-1)).clone();
	 	    		int outSize = 	( (int) (mainFrame.getSize().width*0.5) < mainFrame.getSize().height) ? 					  	(int) (mainFrame.getSize().width*0.5) : mainFrame.getSize().height;
	  		 	out.init(outSize,(int)(outSize*0.8));
	 		}
			out.drawing = true;
			debug ("Setting to stop!");
	 		drawButton.setLabel("Stop");
	 	    	drawThread = new Thread(out,"IFSDraw");
			debug("Starting ...");
	 		drawThread.start();
	 		 
	   	}
	 	else if (source.equals("Load")) loadIFSData(docbasedir+"/"+fileTextField.getText());
		else if (source.equals("Clear")) {
			 if (drawThread != null)
			 	if (drawThread.isAlive()) {
			 		drawThread.stop();
			 		drawButton.setLabel("Draw");
			 	}
			 out.clear();
		}	 
	 	else if (source.equals("Stop")) {
	 		 drawThread.stop();
	 		 drawButton.setLabel("Draw");
	   }
	 	debug("ActionEvent: "+e.getActionCommand());
	 }
	 
	public void windowOpened(WindowEvent e){ }
	public void windowIconified(WindowEvent e){ }
	public void windowDeiconified(WindowEvent e){ }
	public void windowDeactivated(WindowEvent e){ }
	public void windowClosing(WindowEvent e){
		if (drawThread != null)				// Se stava disegnando si ferma
			if (drawThread.isAlive()) {
				drawThread.stop();
			}
			out.clear();

		mainFrame.dispose();
		//destroy();
		//System.exit(0);
	}
	public void windowClosed(WindowEvent e){ }
	public void windowActivated(WindowEvent e){ }


}
}